/* 
 *  mod_cachem.h -- Apache2 DSO shared memory manager module
 * Byron Young
 * 2005
 */

/**
 *@file mod_cachem.h
 *@brief mod_cachem header file.
 *@author Byron Young
 *@version 0.1
 *@bug report to spamiccling@yahoo.com
 *@bug report to bkyoung@users.sourceforge.net
 */

#ifndef MOD_CACHEM_H
#define MOD_CACHEM_H

/**
 *@def CACHEM_INVALID_INDEX
 *@brief An invalid index.
 */
#define CACHEM_INVALID_INDEX (~0)

/**
 *@def CACHEM_INVALID_ID
 *@brief An invalid ID.
 */
#define CACHEM_INVALID_ID (~0)

/**
 *@def CACHEM_NOLOCKING
 *@brief Do not use locking with the function.
 */
#define CACHEM_NOLOCKING 0

/**
 *@def CACHEM_USELOCKING
 *@brief Use locking with the function.
 */
#define CACHEM_USELOCKING 1

#ifndef M_INDEX_T
#define M_INDEX_T
typedef unsigned int m_index_t;
#endif /* M_INDEX_T */

#ifndef CACHEM_ID_T
#define CACHEM_ID_T
typedef unsigned int cachem_id_t;
#endif /* CACHEM_ID_T */

/**
 *@struct cachem_stats
 *@brief The current state of the cache.
 */
typedef struct cachem_stats {
  unsigned int u_blks;
  unsigned int free;
  unsigned int size;
} cachem_stats_t, * p_cachem_stats_t;

/* prototypes */
cachem_id_t cachem_id(const char * s, apr_pool_t * p);
m_index_t cachem_delete(m_index_t index);
void cachem_delete_pid(cachem_id_t id);
m_index_t cachem_add(void * data, unsigned int dsize, cachem_id_t id);
void * cachem_get(m_index_t index, unsigned int * dsize);
m_index_t cachem_get_pid(unsigned int pid, m_index_t index);
m_index_t cachem_get_id(cachem_id_t id, m_index_t index);
apr_status_t cachem_lock();
apr_status_t cachem_unlock();
void cachem_info(p_cachem_stats_t pst);
int cachem_handler_init(request_rec * r);

#endif /* MOD_CACHEM_H */
